@@ -13,6 +13,8 @@ import matplotlib.pyplot as plt
 import pyomo.environ as pe
 import pyomo.opt as po
 
+import ipopt_wrapper
+
 from ase.build import fcc111
 from ase.data import atomic_numbers, covalent_radii
 
@@ -231,7 +233,7 @@ class geom2graph:
             n = set(self.find_nearest_neighbors([i])) - set(self.adsorbate_atoms)
             sum_cn += len(n)
         
-        bot_slab = fcc111(self.surface_type, (10,10,3), periodic = True)
+        bot_slab = fcc111(self.surface_type, (10,10,5), periodic = True)
         cell_center = np.median(bot_slab.positions, axis = 0)
         
         active_site_poss = self.atoms[self.active_site].positions
@@ -349,9 +351,34 @@ class geom2graph:
             for index in model.edge_weights_ads:
                 self.graph.edges[index]['weight'] = int(model.edge_weights_ads[index].value)
                 
-            if self.surface_type in [self.graph.nodes[i]['element'] for i in self.graph.nodes]:
-                for index in model.edge_weights_pts:
-                    self.graph.edges[index]['weight'] = round(model.edge_weights_pts[index].value,3)
+            
+            edges_pt_dict = {i: model.edge_weights_pts[i].value for i in model.edge_weights_pts}          
+            
+            nodes = set([i for sub in model.edges_pts for i in sub])
+            not_pt_nodes = [i for i in nodes if self.symbols[i] != self.surface_type]
+            
+            edge_pt_ads_sort = dict.fromkeys(not_pt_nodes, {})
+            
+            for index in model.edge_weights_pts:
+                intersection = set(index).intersection(not_pt_nodes)
+                if intersection:
+                    edge_pt_ads_sort[list(intersection)[0]][index] = model.edge_weights_pts[index].value
+                    
+            for node in edge_pt_ads_sort:
+                
+                edge_sum = 0
+                edges= 0
+                
+                for edge in edge_pt_ads_sort[node]:
+                    edge_sum += edge_pt_ads_sort[node][edge]
+                    edges += 1
+                edge_sum = edge_sum/edges
+                
+                for edge in edge_pt_ads_sort[node]:
+                    edges_pt_dict[edge] = edge_sum
+            
+            for index in edges_pt_dict: 
+                self.graph.edges[index]['weight'] = round(edges_pt_dict[index],3)
             
             
         else:
@@ -365,25 +392,27 @@ class geom2graph:
         
         node_dict = dict(self.graph.nodes)
     
-        edges_ads, edges_pts, nodes_ads, nodes_pts = {}, {}, {}, {}
+        edges_ads, edges_pts, edges_hs, nodes_ads, nodes_pts, nodes_hs= {}, {}, {}, {}, {}, {}
         
         # Sort edges into surface and adsorbate
         for edge in self.graph.edges:
             
-            if self.symbols[edge[0]] == self.surface_type or self.symbols[edge[1]] == self.surface_type:
-                if self.symbols[edge[0]] != 'H' and self.symbols[edge[1]] != 'H':
+            if 'H' in self.symbols[[edge]]:
+                edges_hs[edge] = self.graph.edges[edge]['weight']
+            
+            elif self.surface_type in self.symbols[[edge]]:
                     edges_pts[edge] = self.graph.edges[edge]['weight']
                     
-                else:
-                    continue
-                    
             else:
                 edges_ads[edge] = self.graph.edges[edge]['weight']
         
         # Sort nodes into surface and adsorbate
         for node in self.graph.nodes:
+            
             if self.symbols[node]==self.surface_type:
                 nodes_pts[node] = node_dict[node]['valency']
+            elif self.symbols[node] == 'H':
+                nodes_hs[node] = node_dict[node]['valency']
             else:
                 nodes_ads[node] = node_dict[node]['valency']
 
@@ -391,12 +420,18 @@ class geom2graph:
         model = pe.ConcreteModel()
         
         model.nodes_ads = pe.Set(initialize = list(set(nodes_ads.keys())))
+        model.nodes_hs = pe.Set(initialize = list(set(nodes_hs.keys())))
         model.edges_ads = pe.Set(initialize = list(set(edges_ads.keys())))
+        model.edges_hs = pe.Set(initialize = list(set(edges_hs.keys())))
         
         model.edge_weights_ads = pe.Var(model.edges_ads, 
                                         within = pe.PositiveIntegers,
                                         initialize = {i: 2 for i in model.edges_ads},
                                         bounds = (1, 4))
+        
+        model.edge_weights_hs = pe.Param(model.edges_hs,
+                                         initialize = {i: 1 for i in model.edges_hs})
+        
         model.valency = pe.Param(model.nodes_ads, 
                                  initialize = nodes_ads, 
                                  within=pe.PositiveIntegers)
@@ -413,52 +448,42 @@ class geom2graph:
         
             def valency_cap(model, i):
             
-                edge_sum = sum([sum([model.edge_weights_ads[j] for j in model.edge_weights_ads if i in j]),
-                                  sum([model.edge_weights_pts[j] for j in model.edge_weights_pts if i in j])])
+                node_in_edge_ads = sum([model.edge_weights_ads[j] for j in model.edge_weights_ads if i in j])
+                node_in_edge_hs = sum([model.edge_weights_hs[j] for j in model.edge_weights_hs if i in j])
+                node_in_edge_pts = sum([model.edge_weights_pts[j] for j in model.edge_weights_pts if i in j])
                 
-                constraint = edge_sum == model.valency[i]
+                return model.valency[i] == node_in_edge_ads + node_in_edge_hs + node_in_edge_pts
                        
-                return constraint
         
-            def tighten_bonds_to_1_1(model):
+            def edge_sum(model):
                 
-                total2 = sum([(model.edge_weights_pts[i]-1)**2 for i in model.edges_pts])
-                total1 = sum([(model.edge_weights_ads[i]-1)**2 for i in model.edges_ads])
-                total= total1+total2
+                total_ads = sum([(model.edge_weights_ads[i]) for i in model.edges_ads])
+                total_pts = sum([(model.edge_weights_pts[i]) for i in model.edges_pts])
+                total = total_ads + total_pts
                 return total
             
-            model.obj = pe.Objective(sense = pe.minimize, rule = tighten_bonds_to_1_1)
-            model.con = pe.Constraint(model.nodes_ads, rule = valency_cap)
+
             
         else:
             
-            # def valency_cap(model, i):
+            def valency_cap(model, i):
             
-            #     edge_sum = sum([model.edge_weights_ads[j] for j in model.edge_weights_ads if i in j])
-                                 
-            #     constraint = edge_sum == model.valency[i]
-                       
-            #     return constraint
-            
-            # def valency_cap(model, i):
-            
-            #     edge_sum = sum([model.edge_weights_ads[j] for j in model.edge_weights_ads if i in j])
-            #     constraint = edge_sum == model.valency[i]
-                       
-            #     return constraint
-            
-            def tighten_bonds_to_1_1(model):
+                node_in_edge_ads = sum([model.edge_weights_ads[j] for j in model.edge_weights_ads if i in j])
+                node_in_edge_hs = sum([model.edge_weights_hs[j] for j in model.edge_weights_hs if i in j])
                 
-                total = sum([(model.edge_weights_ads[i]-1)**2 for i in model.edges_ads])
-                return total
+                return model.valency[i] == node_in_edge_ads + node_in_edge_hs
+                       
+            
+            def edge_sum(model):
+                
+                return sum([(model.edge_weights_ads[i]) for i in model.edges_ads])
 
 
-            model.obj = pe.Objective(sense = pe.minimize, rule = tighten_bonds_to_1_1)
-            # model.con = pe.Constraint(model.nodes_ads, rule = valency_cap)
+
+        model.obj = pe.Objective(sense = pe.maximize, rule = edge_sum)
+        model.con = pe.Constraint(model.nodes_ads, rule = valency_cap)
             
-            
-            
-        solver = po.SolverFactory('ipopt')
+        solver = po.SolverFactory('glpk')
         
         # try:
         print('Solving ' + str(self.atoms[self.adsorbate_atoms].symbols))
@@ -516,10 +541,9 @@ class geom2graph:
         nx.draw_networkx_edge_labels(self.graph, pos, edge_labels)
         plt.show()
         
-    def get_SMILES_string(self):
-        
-        self.smiles = pysmiles.write_smiles(self.graph)
-        print(self.smiles)
+    def get_SMILES_string(self):    
+        self.smiles_string = pysmiles.write_smiles(self.graph)
+
         
     def generalize_graph_metal(self):
         if not self.graph:
@@ -530,19 +554,22 @@ class geom2graph:
             if self.graph.nodes[n[0]]['element'] == self.surface_type:
                 self.graph.nodes[n[0]]['element']='M'
         
-# if __name__=="__main__":
-#     # vasp = geom2graph(vasp = 'vasp_to_network_tests/CONTCARmethane')
-#     vasp = geom2graph(vasp = 'vasp_to_network_tests/CONTCARpraneetalphahdown')
-#     vasp = geom2graph(vasp = 'vasp_to_network_tests/CONTCARCCH2CH3ont')
+if __name__=="__main__":
+    # vasp = geom2graph(vasp = 'vasp_to_network_tests/CONTCARmethane')
+    #vasp = geom2graph(vasp = 'Examples/ase_to_graph/CONTCARpraneetalphahdown')
+    vasp = geom2graph(vasp = 'Examples/ase_to_graph/CONTCAR_CHCH_holhol')
     
-#     # vasp.find_active_site_x_nearest_neighbors(2)
+    # vasp.find_active_site_x_nearest_neighbors(2)
     
-#     # for i in vasp.active_site_nn[2]:
-#     #     vasp.atoms[i].symbol = 'Al'
-#     # for i in vasp.active_site_nn[1]:
-#     #     vasp.atoms[i].symbol = 'Au'
-#     # vasp.add_h_surf_bonds()
-#     vasp.gen_graph()
-#     # vasp.get_SMILES_string()
-#     # vasp.get_gcn()
-#     vasp.draw_graph()
+    # for i in vasp.active_site_nn[2]:
+    #     vasp.atoms[i].symbol = 'Al'
+    # for i in vasp.active_site_nn[1]:
+    #     vasp.atoms[i].symbol = 'Au'
+    vasp.add_h_surf_bonds()
+    vasp.gen_graph()
+    vasp.get_SMILES_string()
+    print(vasp.smiles_string)
+    vasp.get_gcn()
+    print(vasp.gcn)
+    
+    vasp.draw_graph()
